@@ -30,7 +30,6 @@ from ._exceptions import LoadError, LoadErrorReason, BstError
 from ._message import Message, MessageType
 from ._profile import Topics, profile_start, profile_end
 from ._artifactcache import ArtifactCache, ArtifactCacheUsage
-from ._artifactcache.cascache import CASCache
 from ._workspaces import Workspaces
 from .plugin import Plugin
 
@@ -147,10 +146,15 @@ class Context():
         # a $XDG_CONFIG_HOME/buildstream.conf file
         #
         if not config:
-            default_config = os.path.join(os.environ['XDG_CONFIG_HOME'],
-                                          'buildstream.conf')
-            if os.path.exists(default_config):
-                config = default_config
+            #
+            # Support parallel installations of BuildStream by first
+            # trying buildstream1.conf and then falling back to buildstream.conf.
+            #
+            for config_filename in ("buildstream1.conf", "buildstream.conf"):
+                default_config = os.path.join(os.environ["XDG_CONFIG_HOME"], config_filename)
+                if os.path.exists(default_config):
+                    config = default_config
+                    break
 
         # Load default config
         #
@@ -241,7 +245,7 @@ class Context():
     @property
     def artifactcache(self):
         if not self._artifactcache:
-            self._artifactcache = CASCache(self)
+            self._artifactcache = ArtifactCache(self)
 
         return self._artifactcache
 
