@@ -40,7 +40,7 @@ from .._message import Message, MessageType, unconditional_messages
 from .._stream import Stream
 from .._versions import BST_FORMAT_VERSION
 from .. import _yaml
-from .._scheduler import ElementJob
+from .._scheduler import ElementJob, JobStatus
 
 # Import frontend assets
 from . import Profile, LogLine, Status
@@ -192,6 +192,7 @@ class App():
             'fetchers': 'sched_fetchers',
             'builders': 'sched_builders',
             'pushers': 'sched_pushers',
+            'max_jobs': 'build_max_jobs',
             'network_retries': 'sched_network_retries'
         }
         for cli_option, context_attr in override_map.items():
@@ -199,7 +200,7 @@ class App():
             if option_value is not None:
                 setattr(self.context, context_attr, option_value)
         try:
-            Platform.create_instance(self.context)
+            Platform.get_platform()
         except BstError as e:
             self._error_exit(e, "Error instantiating platform")
 
@@ -215,6 +216,13 @@ class App():
         # Propagate pipeline feedback to the user
         self.context.set_message_handler(self._message_handler)
 
+        # Preflight the artifact cache after initializing logging,
+        # this can cause messages to be emitted.
+        try:
+            self.context.artifactcache.preflight()
+        except BstError as e:
+            self._error_exit(e, "Error instantiating artifact cache")
+
         #
         # Load the Project
         #
@@ -496,7 +504,10 @@ class App():
                 choice = click.prompt("Choice:",
                                       value_proc=_prefix_choice_value_proc(['continue', 'quit', 'terminate']),
                                       default='continue', err=True)
-            except click.Abort:
+            except (click.Abort, SystemError):
+                # In some cases, the readline buffer underlying the prompt gets corrupted on the second CTRL+C
+                # This throws a SystemError, which doesn't seem to be problematic for the rest of the program
+
                 # Ensure a newline after automatically printed '^C'
                 click.echo("", err=True)
                 choice = 'terminate'
@@ -518,20 +529,20 @@ class App():
         self._status.add_job(job)
         self._maybe_render_status()
 
-    def _job_completed(self, job, success):
+    def _job_completed(self, job, status):
         self._status.remove_job(job)
         self._maybe_render_status()
 
         # Dont attempt to handle a failure if the user has already opted to
         # terminate
-        if not success and not self.stream.terminated:
+        if status == JobStatus.FAIL and not self.stream.terminated:
 
             if isinstance(job, ElementJob):
                 element = job.element
                 queue = job.queue
 
                 # Get the last failure message for additional context
-                failure = self._fail_messages.get(element._get_unique_id())
+                failure = self._fail_messages.get(element._unique_id)
 
                 # XXX This is dangerous, sometimes we get the job completed *before*
                 # the failure message reaches us ??
@@ -591,7 +602,10 @@ class App():
                 try:
                     choice = click.prompt("Choice:", default='continue', err=True,
                                           value_proc=_prefix_choice_value_proc(choices))
-                except click.Abort:
+                except (click.Abort, SystemError):
+                    # In some cases, the readline buffer underlying the prompt gets corrupted on the second CTRL+C
+                    # This throws a SystemError, which doesn't seem to be problematic for the rest of the program
+
                     # Ensure a newline after automatically printed '^C'
                     click.echo("", err=True)
                     choice = 'terminate'
