@@ -1,5 +1,6 @@
 import os
 import sys
+import fcntl
 
 import click
 from .. import _yaml
@@ -158,6 +159,17 @@ def override_main(self, args=None, prog_name=None, complete_var=None,
         # Regular client return for test cases
         return
 
+    # Check output file descriptor at earliest opportunity, to
+    # provide a reasonable error message instead of a stack trace
+    # in the case that it is blocking
+    for stream in (sys.stdout, sys.stderr):
+        fileno = stream.fileno()
+        flags = fcntl.fcntl(fileno, fcntl.F_GETFL)
+        if flags & os.O_NONBLOCK:
+            click.echo("{} is currently set to O_NONBLOCK, try opening a new shell"
+                       .format(stream.name), err=True)
+            sys.exit(-1)
+
     original_main(self, args=args, prog_name=prog_name, complete_var=None,
                   standalone_mode=standalone_mode, **extra)
 
@@ -196,6 +208,8 @@ def print_version(ctx, param, value):
               help="Maximum simultaneous build tasks")
 @click.option('--pushers', type=click.INT, default=None,
               help="Maximum simultaneous upload tasks")
+@click.option('--max-jobs', type=click.INT, default=None,
+              help="Number of parallel jobs allowed for a given build task")
 @click.option('--network-retries', type=click.INT, default=None,
               help="Maximum retries for network tasks")
 @click.option('--no-interactive', is_flag=True, default=False,
