@@ -221,18 +221,17 @@ class Project():
     # Instantiate and return an element
     #
     # Args:
-    #    artifacts (ArtifactCache): The artifact cache
     #    meta (MetaElement): The loaded MetaElement
     #    first_pass (bool): Whether to use first pass configuration (for junctions)
     #
     # Returns:
     #    (Element): A newly created Element object of the appropriate kind
     #
-    def create_element(self, artifacts, meta, *, first_pass=False):
+    def create_element(self, meta, *, first_pass=False):
         if first_pass:
-            return self.first_pass_config.element_factory.create(self._context, self, artifacts, meta)
+            return self.first_pass_config.element_factory.create(self._context, self, meta)
         else:
-            return self.config.element_factory.create(self._context, self, artifacts, meta)
+            return self.config.element_factory.create(self._context, self, meta)
 
     # create_source()
     #
@@ -302,7 +301,6 @@ class Project():
     #
     # Args:
     #    targets (list): Target names
-    #    artifacts (ArtifactCache): Artifact cache
     #    rewritable (bool): Whether the loaded files should be rewritable
     #                       this is a bit more expensive due to deep copies
     #    fetch_subprojects (bool): Whether we should fetch subprojects as a part of the
@@ -311,7 +309,7 @@ class Project():
     # Returns:
     #    (list): A list of loaded Element
     #
-    def load_elements(self, targets, artifacts, *,
+    def load_elements(self, targets, *,
                       rewritable=False, fetch_subprojects=False):
         with self._context.timed_activity("Loading elements", silent_nested=True):
             meta_elements = self.loader.load(targets, rewritable=rewritable,
@@ -320,7 +318,7 @@ class Project():
 
         with self._context.timed_activity("Resolving elements"):
             elements = [
-                Element._new_from_meta(meta, artifacts)
+                Element._new_from_meta(meta)
                 for meta in meta_elements
             ]
 
@@ -401,6 +399,37 @@ class Project():
                 "Project requested format version {}, but BuildStream {}.{} only supports up until format version {}"
                 .format(format_version, major, minor, BST_FORMAT_VERSION))
 
+        # Since BuildStream 2, project.conf is required to specify min-version.
+        #
+        # Detect this and raise an error, indicating which major version of BuildStream
+        # should be used for this project.
+        #
+        min_version = _yaml.node_get(pre_config_node, str, 'min-version', default_value=None)
+        if min_version:
+
+            # Handle case of malformed min-version
+            #
+            try:
+                major, minor = utils._parse_version(min_version)
+            except UtilError as e:
+                raise LoadError(
+                    LoadErrorReason.UNSUPPORTED_PROJECT,
+                    "This is not a BuildStream 1 project: {}".format(e)
+                ) from e
+
+            # Raise a helpful error indicating what the user should do to
+            # use this project.
+            #
+            raise LoadError(
+                LoadErrorReason.UNSUPPORTED_PROJECT,
+                "Tried to load a BuildStream {} project with BuildStream 1".format(major),
+
+                # TODO: Include a link to the appropriate documentation for parallel
+                # installing different BuildStream versions.
+                #
+                detail="Please install at least BuildStream {}.{} to use this project".format(major, minor)
+            )
+
         # FIXME:
         #
         #   Performing this check manually in the absense
@@ -436,7 +465,7 @@ class Project():
         self._project_includes = Includes(self.loader, copy_tree=False)
 
         project_conf_first_pass = _yaml.node_copy(self._project_conf)
-        self._project_includes.process(project_conf_first_pass, only_local=True)
+        self._project_includes.process(project_conf_first_pass, only_local=True, process_project_options=False)
         config_no_include = _yaml.node_copy(self._default_config_node)
         _yaml.composite(config_no_include, project_conf_first_pass)
 
@@ -460,7 +489,7 @@ class Project():
     #
     def _load_second_pass(self):
         project_conf_second_pass = _yaml.node_copy(self._project_conf)
-        self._project_includes.process(project_conf_second_pass)
+        self._project_includes.process(project_conf_second_pass, process_project_options=False)
         config = _yaml.node_copy(self._default_config_node)
         _yaml.composite(config, project_conf_second_pass)
 
